import 'regenerator-runtime/runtime'
import path from 'path'
import dotenv from 'dotenv'

dotenv.config({ path: path.join(__dirname, '../.env') })

import http from 'http'
import express from 'express'
import proxy from 'express-http-proxy'
import bodyParser from 'body-parser'
import cookieParser from 'cookie-parser'
import session from 'express-session'
import redis from 'redis'
import connectRedis from 'connect-redis'

import connectSocketIO from 'socket.io'
import passportSocketIo from 'passport.socketio'

import morgan from 'morgan'

import logger, { morganStream } from './logger'

import passport from 'passport'
import mongoose from 'mongoose'

import { authRouter, initPassport } from './auth/passport'

const RedisStore = connectRedis(session)
const redisClient = redis.createClient({
  host: '127.0.0.1',
  port: 6379,

  password: process.env.REDIS_PASSWORD,
})

redisClient.on('error', (error) => {
  console.error(error)
})

redisClient.on('connect', () => {
  console.log('Connection to Redis has been established successfully.')
})

mongoose.connect(
  'mongodb://localhost:27017',
  {
    useCreateIndex: true,
    useNewUrlParser: true,
    useUnifiedTopology: true,
    user: process.env.MONGODB_USER,
    pass: process.env.MONGODB_PASSWORD,
    dbName: 'museum',
  },
  () => {
    console.log('Connection to MongoDB has been established successfully.')
  },
)

const app = express()
app.set('trust proxy', 1)
const server = http.createServer(app)
// @ts-ignore
const io = connectSocketIO(server)

app.use(bodyParser.json())
app.use(bodyParser.urlencoded({ extended: false }))
app.use(cookieParser(process.env.SESSION_SECRET))
app.use(
  session({
    store: new RedisStore({ client: redisClient }),
    secret: process.env.SESSION_SECRET,
    resave: false,
    saveUninitialized: false,
  }),
)

app.use(passport.initialize())
app.use(passport.session())
initPassport()
app.use('/auth', authRouter)

app.use(morgan('short', { stream: morganStream }))

function onAuthorizeSuccess(data, accept) {
  // console.log(data.user)
  logger.debug(
    `Successful connection to socket.io from ${data.user._id} (${data.user.email})`,
  )
  accept(null, true)
}

function onAuthorizeFail(_, message, error, accept) {
  if (error) throw new Error(message)
  logger.debug('failed connection to socket.io:', message)
  accept(null, false)
}

io.use(
  passportSocketIo.authorize({
    // @ts-ignore
    cookieParser: cookieParser, // the same middleware you registrer in express
    secret: process.env.SESSION_SECRET, // the session_secret to parse the cookie
    store: new RedisStore({ client: redisClient }), // we NEED to use a sessionstore. no memorystore please
    success: onAuthorizeSuccess, // *optional* callback on success - read more below
    fail: onAuthorizeFail, // *optional* callback on fail/error - read more below
  }),
)

io.on('connection', (socket) => {
  logger.debug('a user connected')
  //console.log(socket.request.user)
  socket.on('disconnect', () => {
    logger.debug('a user connected')
  })
})

if (process.env.NODE_ENV !== 'PRODUCTION') {
  app.use('/', proxy('http://localhost:4000/'))
} else {
  // probably serve up build version in production
}

server.listen(3000, () => {
  console.log('> Ready on http://localhost:3000')
})
